/*
 * Implementacija pomocu binaranog stabla pretrage.
 */
 
#include <stdio.h>
#include <stdlib.h>

typedef int tipelementa;

typedef struct _cvor { 
        tipelementa element; 
        struct _cvor *levo, *desno; 
} cvor;

typedef cvor *skup;


void inorder(skup r, int dubina) {
     if(r) {
        int i;
        inorder(r->levo, dubina + 1);
        for(i = 0; i < dubina; i++) {
            printf("-");
        }
        printf("%d\n", r->element);
        inorder(r->desno, dubina + 1);
     }
}

void insert(tipelementa x, skup *r) {
     if(*r == NULL) {
         // Stablo je prazno, ili smo nasli mesto za cvor sa elementom x.
         *r = (cvor*)malloc(sizeof(cvor));
         (*r)->element = x;
         (*r)->levo = NULL;
         (*r)->desno = NULL;
     }
     else if((*r)->element == x) {
          // Ne radi nista, element je vec u skupu.
          return;
     }
     else if((*r)->element < x) {
          insert(x, &((*r)->desno));
     }
     else {
          insert(x, &((*r)->levo));
     }
}

tipelementa delete_min(skup *r) {
    if ((*r)->levo == NULL) {
        // *r je krajnji levi -> najmanji
        // Cuva se vrednost cvora.
        tipelementa min_element = (*r)->element;
        
        // Kao i pokazivac na cvor zbog kasnijeg brisanja.
        cvor *temp = *r; 
        *r = (*r)->desno; 
        free(temp);
        return min_element;
    }
    else {
        return delete_min(&((*r)->levo));
    }
}

void delete(tipelementa x, skup *r) {
     if (*r != NULL) {
        if(x < (*r)->element) {
            delete( x, &((*r)->levo) );
        }
        else if (x > (*r)->element) {
            delete( x, &((*r)->desno) );
        }
        else {
             // Trazeni cvor je pronadjen, sledi brisanje.
             if ( ((*r)->levo == NULL) && ((*r)->desno == NULL)) {
                // list je, treba ga samo obrisati
                 free(*r); 
                 *r = NULL;
             }
             else if ((*r)->levo == NULL) {
                 // Ima samo desno dete, zameniti ga sa njim
                 cvor *temp = *r; 
                 *r = (*r)->desno; 
                 free(temp); 
             }
             else if ((*r)->desno == NULL ) {
                 // Ima samo levo dete, zameniti ga sa njim
                 cvor *temp = *r; 
                 *r = (*r)->levo; 
                 free(temp); 
             }
             else {
                   // Ima samo oba deteta, naci najmanje dete u desnom
                   // podstabli i postaviti ga na mesto vrednosti cvora
                   // koji se brise.
                  (*r)->element = delete_min ( &((*r)->desno) );
             }
             
        }
     }
}

int member(tipelementa x, skup r) {
     if(!r) {
         return 0;
     }
     
     if(r->element == x) {
         return 1;
     }
     else if(r->element < x) {
          return member(x, r->desno);
     }
     else {
          return member(x, r->levo);
     }
}

void obrisi(skup *r) {
     if(*r) {
           obrisi(&((*r)->levo));
           obrisi(&((*r)->desno));
           
           free(*r);
           *r = NULL;
     }
}

int main() {
    skup r = NULL;
    insert(10, &r);
    insert(5, &r);
    insert(14, &r);
    insert(7, &r);
    insert(12, &r);
    insert(18, &r);
    insert(15, &r);
    
    inorder(r, 0);
    
    delete(10, &r); // brisanje korena, sa oba deteta
    
    printf("Nakon brisanja:\n");
    inorder(r, 0);
    
    obrisi(&r);
    
    system("pause");
    return 0;
}

